@@ -3,7 +3,7 @@ package Beekeeper::JSONRPC::Notification;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
 
 sub new {
@@ -22,7 +22,7 @@ sub params { $_[0]->{params} }
 sub id     { undef           }
 
 sub mqtt_properties {
-    $_[0]->{_mqtt_properties};
+    $_[0]->{_mqtt_prop};
 }
 
 1;
@@ -39,34 +39,27 @@ Beekeeper::JSONRPC::Notification - Representation of a JSON-RPC notification.
  
 =head1 VERSION
  
-Version 0.06
+Version 0.03
 
 =head1 DESCRIPTION
 
-Objects of this class represent a JSON-RPC notification (see L<http://www.jsonrpc.org/specification>).
-
-On worker classes the method handlers setted by L<Beekeeper::Worker::accept_notifications> 
-will receive these objects as parameters.
+Objects of this class represents a JSON-RPC notification (see L<http://www.jsonrpc.org/specification>).
 
 =head1 ACCESSORS
 
-=over
+=over 4
 
 =item method
 
-Returns a string with the name of the method invoked.
+A string with the name of the method to be invoked.
 
 =item params
 
-Returns the arbitrary data structure passed as parameters.
+An arbitrary data structure to be passed as parameters to the defined method.
 
 =item id
 
-Always returns undef.
-
-=item mqtt_properties
-
-Returns a hashref containing the MQTT properties of the notification.
+It is always undef.
 
 =back
 
