@@ -3,11 +3,10 @@ package Beekeeper::MQTT;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
 use AnyEvent;
 use AnyEvent::Handle;
-use Time::HiRes;
 use List::Util 'shuffle';
 use Scalar::Util 'weaken';
 use Exporter 'import';
@@ -262,8 +261,6 @@ sub _fatal {
     $self->{error_cb}->($errstr);
 }
 
-our $BUSY_SINCE = undef;
-our $BUSY_TIME  = 0;
 
 sub connect {
     my ($self, %args) = @_;
@@ -369,19 +366,11 @@ sub _connect {
             my $offs;
             my $byte;
 
-            my $timing_packets;
-
-            unless (defined $BUSY_SINCE) {
-                # Measure time elapsed while processing incoming packets
-                $BUSY_SINCE = Time::HiRes::time;
-                $timing_packets = 1; 
-            }
-
             PARSE_PACKET: {
 
                 $rbuff_len = length $fh->{rbuf};
 
-                last PARSE_PACKET unless $rbuff_len >= 2;
+                return unless $rbuff_len >= 2;
 
                 unless ($packet_type) {
 
@@ -393,7 +382,7 @@ sub _connect {
                         $byte = unpack "C", substr( $fh->{rbuf}, $offs++, 1 );
                         $packet_len += ($byte & 0x7f) * $mult;
                         last unless ($byte & 0x80);
-                        last PARSE_PACKET if ($offs >= $rbuff_len); # Not enough data
+                        return if ($offs >= $rbuff_len); # Not enough data
                         $mult *= 128;
                         redo if ($offs < 5);
                     }
@@ -407,7 +396,7 @@ sub _connect {
 
                 if ($rbuff_len < ($offs + $packet_len)) {
                     # Not enough data
-                    last PARSE_PACKET;
+                    return;
                 }
 
                 # Consume packet from buffer
@@ -475,11 +464,6 @@ sub _connect {
                 # Handle could have been destroyed at this point
                 redo PARSE_PACKET if defined $fh->{rbuf};
             }
-
-            if (defined $timing_packets) {
-                $BUSY_TIME += Time::HiRes::time - $BUSY_SINCE;
-                undef $BUSY_SINCE;
-            }
         },
     );
 
@@ -941,7 +925,7 @@ sub subscribe {
     }
 
     my $packet_id = $self->{packet_seq}++;
-    $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
+    $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
 
     # Set callback for incomings PUBLISH
     my $subscr_id = $self->{subscr_seq}++;
@@ -1092,7 +1076,7 @@ sub unsubscribe {
     }
 
     my $packet_id = $self->{packet_seq}++;
-    $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
+    $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
 
     # Set callback for UNSUBACK
     $self->{packet_cb}->{$packet_id} = {
@@ -1258,7 +1242,7 @@ sub publish {
     my $packet_id;
     if ($qos) {
         $packet_id = $self->{packet_seq}++;
-        $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
+        $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
     }
 
     my $raw_prop = '';
@@ -1353,7 +1337,7 @@ sub publish {
         # As client may be syncronous, wait until entire message is sent.
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
+        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1733,7 +1717,7 @@ sub flush_buffer {
         # Kernel write buffer is full, see publish() above
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
+        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1784,7 +1768,7 @@ Beekeeper::MQTT - Asynchronous MQTT 5.0 client.
  
 =head1 VERSION
  
-Version 0.06
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -1821,7 +1805,7 @@ Version 0.06
   
   $mqtt->disconnect;
 
-Most methods allow to send arbitrary properties as key-value pairs of utf8 strings.
+Most methods allows to send arbitrary properties along with commands.
 
 Except for trivial cases, error checking is delegated to the server.
 
@@ -1835,7 +1819,7 @@ The MQTT specification can be found at L<https://mqtt.org/mqtt-specification>
 
 =head3 new ( %options )
 
-=over
+=over 4
 
 =item host
 
@@ -1878,7 +1862,7 @@ connection when this is called.
 
 Connect to the MQTT server and do handshake. On failure retries until timeout.
 
-=over
+=over 4
 
 =item blocking => $bool
 
@@ -1893,9 +1877,12 @@ Callback which is executed after the server accepted the connection.
 
 =head3 disconnect ( %args )
 
-Does a graceful disconnection from the server.
+A client can disconnect from the server at anytime by closing the socket but 
+there is no guarantee that the previously sent packets have been received by
+the server. This method should be called to do a graceful shutdown, where the
+client is assured that all previous packets have been received by the server.
 
-=over
+=over 4
 
 =item $reason_code
 
@@ -1918,7 +1905,7 @@ it will be passed to given on_publish callback:
       },
   );
 
-=over
+=over 4
 
 =item topics => \@topics
 
@@ -1950,7 +1937,7 @@ from that topic. Example:
       topics => ['topic/foo','topic/bar']
   );
 
-=over
+=over 4
 
 =item topics => \@topics
 
@@ -1976,7 +1963,7 @@ Sends a message to a topic. Example:
       }
   );
 
-=over
+=over 4
 
 =item topic => $str
 
@@ -2025,7 +2012,7 @@ of utf8 strings.
 Used to acknowledge the receipt of a message received from a subscription with QoS 1.
 The server should resend the message until it is acknowledged.
 
-=over
+=over 4
 
 =item packet_id => $int
 
