@@ -3,8 +3,9 @@ package Beekeeper::Service::Router::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
+use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -142,7 +143,7 @@ sub on_shutdown {
     my $tmr = AnyEvent->timer( after => 30, cb => sub { $cv->send });
     $cv->recv;
 
-    # 4. Just in case of pool full stop, wait for workers to finish their current tasks
+    # 4. Just in case of pool stop, wait for workers to finish their current jobs
     my $wait = AnyEvent->condvar;
     $tmr = AnyEvent->timer( after => SHUTDOWN_WAIT, cb => sub { $wait->send });
     $wait->recv;
@@ -205,7 +206,7 @@ sub pull_frontend_requests {
 
         my $src_queue = "\$share/BKPR/req/$backend_role-$lane";
 
-        my ($payload_ref, $mqtt_properties);
+        my ($payload_ref, $msg_prop);
         my ($dest_queue, $reply_to, $caller_id, $mqtt_session);
         my %pub_args;
 
@@ -213,16 +214,16 @@ sub pull_frontend_requests {
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $mqtt_properties) = @_;
+                ($payload_ref, $msg_prop) = @_;
 
                 # (!) UNTRUSTED REQUEST
 
                 # eg: req/backend/myapp/service
-                $dest_queue = $mqtt_properties->{'fwd_to'} || '';
+                $dest_queue = $msg_prop->{'fwd_to'} || '';
                 return unless $dest_queue =~ m|^req(/(?!_)[\w-]+)+$|;
 
                 # eg: priv/7nXDsxMDwgLUSedX
-                $reply_to = $mqtt_properties->{'response_topic'} || '';
+                $reply_to = $msg_prop->{'response_topic'} || '';
                 return unless $reply_to =~ m|^priv/(\w{16,23})$|;
                 $caller_id = $1;
 
@@ -275,15 +276,15 @@ sub pull_backend_responses {
 
         my $src_queue = "\$share/BKPR/res/$frontend_id-$lane";
 
-        my ($payload_ref, $mqtt_properties, $dest_queue);
+        my ($payload_ref, $msg_prop, $dest_queue);
 
         $backend_bus->subscribe(
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $mqtt_properties) = @_;
+                ($payload_ref, $msg_prop) = @_;
 
-                ($dest_queue) = split('@', $mqtt_properties->{'addr'}, 2);
+                ($dest_queue) = split('@', $msg_prop->{'addr'}, 2);
 
                 $frontend_bus->publish(
                     topic   => $dest_queue,
@@ -327,15 +328,15 @@ sub pull_backend_notifications {
 
         my $src_queue = "\$share/BKPR/msg/$frontend_role-$lane",
 
-        my ($payload_ref, $mqtt_properties, $destination, $address);
+        my ($payload_ref, $msg_prop, $destination, $address);
 
         $backend_bus->subscribe(
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $mqtt_properties) = @_;
+                ($payload_ref, $msg_prop) = @_;
 
-                ($destination, $address) = split('@', $mqtt_properties->{'fwd_to'}, 2);
+                ($destination, $address) = split('@', $msg_prop->{'fwd_to'}, 2);
 
                 if (defined $address) {
 
@@ -535,7 +536,7 @@ Beekeeper::Service::Router::Worker - Route messages between backend and frontend
 
 =head1 VERSION
  
-Version 0.06
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -557,6 +558,11 @@ routers will need the in-memory shared table, and this shared table will not sca
 well as the rest of the system. So a better strategy (some kind of partition) will 
 be needed for applications with a large number of concurrent clients.
 
+=head1 TODO
+
+Measure and report the percentage of busy time ('load' in bkpr-top). This metric is essential
+to assess the routing capacity of a given system.
+
 =head1 SEE ALSO
  
 L<Beekeeper::Service::Router>
