@@ -3,8 +3,9 @@ package Beekeeper::Service::Sinkhole::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
+use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -20,7 +21,7 @@ sub authorize_request {
         return unless $self->__has_authorization_token('BKPR_SYSTEM');
     }
 
-    # All requests will be rejected later by reject_request 
+    # All requests will be rejected by reject_job 
     return BKPR_REQUEST_AUTHORIZED;
 }
 
@@ -70,7 +71,7 @@ sub on_unserviced_queues {
         my $local_bus = $self->{_BUS}->{bus_role};
         log_error "Draining unserviced req/$local_bus/$queue";
 
-        $self->accept_remote_calls( "$queue.*" => 'reject_request' );
+        $self->accept_remote_calls( "$queue.*" => 'reject_job' );
     }
 }
 
@@ -97,12 +98,12 @@ sub on_worker_status {
     }
 }
 
-sub reject_request {
+sub reject_job {
     my ($self, $params, $req) = @_;
 
     # Just return a JSONRPC error response
 
-    if ($req->mqtt_properties->{'auth'}) {
+    if ($req->get_auth_tokens) {
         # When client provided some kind of authentication tell him the truth
         # about the service being down. Otherwise the one trying to fix the 
         # issue may be deceived into looking for auth/permissions problems
@@ -127,7 +128,7 @@ Beekeeper::Service::Sinkhole::Worker - Handle unserviced call topics
 
 =head1 VERSION
  
-Version 0.06
+Version 0.03
 
 =head1 DESCRIPTION
 
@@ -144,10 +145,8 @@ receive an error response instead of timing out.
 As soon as a worker of the downed service becomes online again the Sinkhole
 workers will stop rejecting requests.
 
-Sinkhole workers are not created automatically. In order to add Sinkhole workers
-to a pool these must be declared into config file C<pool.config.json>.
-
-A single Sinkhole worker can handle around 3500 req/s.
+A Sinkhole worker is created automatically in every worker pool, and it can 
+handle around 500 req/s. Extra workers can simply be declared into config file.
 
 =head1 AUTHOR
 
