@@ -3,8 +3,9 @@ package Beekeeper::Service::ToyBroker::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
+use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -106,7 +107,7 @@ sub start_listener {
     my ($self, $listener) = @_;
     weaken($self);
 
-    my $max_packet_size = $listener->{'max_packet_size'};
+    my $max_packet_size = $listener->{'max_packet_size'} || 65536;
 
     my $addr = $listener->{'listen_addr'} || '127.0.0.1';  # Must be an IPv4 or IPv6 address
     my $port = $listener->{'listen_port'} ||  1883;
@@ -156,7 +157,7 @@ sub start_listener {
                             redo if ($offs < 5);
                         }
 
-                        if ($max_packet_size && $packet_len > $max_packet_size) {
+                        if ($packet_len > $max_packet_size) {
                             $self->disconnect($fh, reason_code => 0x95);
                             return;
                         }
@@ -1264,8 +1265,7 @@ sub unsubscribe_client {
 }
 
 
-package
-    Beekeeper::Service::ToyBroker::Client;   # hide from PAUSE
+package Beekeeper::Service::ToyBroker::Client;
 
 sub new {
     my ($class, %args) = @_;
@@ -1331,8 +1331,7 @@ sub resend_unacked_messages {
 }
 
 
-package
-    Beekeeper::Service::ToyBroker::TopicFilter;   # hide from PAUSE
+package Beekeeper::Service::ToyBroker::TopicFilter;
 
 sub new {
     my ($class, $topic_filter) = @_;
@@ -1398,8 +1397,7 @@ sub remove_shared_subscription {
 }
 
 
-package
-    Beekeeper::Service::ToyBroker::Subscription;   # hide from PAUSE
+package Beekeeper::Service::ToyBroker::Subscription;
 
 sub new {
     my $class = shift;
@@ -1438,8 +1436,7 @@ sub send_message {
 }
 
 
-package
-    Beekeeper::Service::ToyBroker::SharedSubscription;   # hide from PAUSE
+package Beekeeper::Service::ToyBroker::SharedSubscription;
 
 sub new {
     my ($class, %args) = @_;
@@ -1508,7 +1505,7 @@ Beekeeper::Service::ToyBroker::Worker - Basic MQTT 5 broker
 
 =head1 VERSION
 
-Version 0.06
+Version 0.03
 
 =head1 DESCRIPTION
 
