@@ -3,7 +3,7 @@ package Beekeeper::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.03';
 
 use Beekeeper::Client ':worker';
 use Beekeeper::Logger ':log_levels';
@@ -16,6 +16,8 @@ use Digest::SHA 'sha256_hex';
 use Scalar::Util 'blessed';
 use Carp;
 
+#TODO: our @CARP_NOT = ('AnyEvent', 'Beekeeper::MQTT');
+
 use constant COMPILE_ERROR_EXIT_CODE => 99;
 use constant REPORT_STATUS_PERIOD    => 5;
 use constant UNSUBSCRIBE_LINGER      => 2;
@@ -54,9 +56,6 @@ sub log_info     (@) { $LogLevel >= LOG_INFO   && $Logger->( LOG_INFO,   @_ ) }
 sub log_debug    (@) { $LogLevel >= LOG_DEBUG  && $Logger->( LOG_DEBUG,  @_ ) }
 sub log_trace    (@) { $LogLevel >= LOG_TRACE  && $Logger->( LOG_TRACE,  @_ ) }
 
-our $BUSY_SINCE; *BUSY_SINCE = \$Beekeeper::MQTT::BUSY_SINCE;
-our $BUSY_TIME;  *BUSY_TIME  = \$Beekeeper::MQTT::BUSY_TIME;
-
 my %AUTH_TOKENS;
 my $JSON;
 
@@ -87,14 +86,14 @@ sub new {
         hostname        => hostname(),
         stop_cv         => undef,
         callbacks       => {},
-        task_queue_high => [],
-        task_queue_low  => [],
+        job_queue_high  => [],
+        job_queue_low   => [],
         queued_tasks    => 0,
-        in_progress     => 0,
         last_report     => 0,
-        calls_count     => 0,
+        calls_count      => 0,
         notif_count     => 0,
         busy_time       => 0,
+        busy_since      => 0,
     };
 
     $JSON = JSON::XS->new;
@@ -140,15 +139,11 @@ sub new {
 sub __init_auth_tokens {
     my ($self) = @_;
 
-    # Using a hashing function makes harder to access the wrong worker pool by mistake,
-    # but it is not an effective access restriction: anyone with access to the backend
-    # bus credentials can easily inspect and clone auth data tokens
+    my $secret = 'salt'; #TODO: read from config file
 
-    my $salt = $self->{_CLIENT}->{auth_salt} || '';
-
-    $AUTH_TOKENS{'BKPR_SYSTEM'} = sha256_hex('BKPR_SYSTEM'. $salt);
-    $AUTH_TOKENS{'BKPR_ADMIN'}  = sha256_hex('BKPR_ADMIN' . $salt);
-    $AUTH_TOKENS{'BKPR_ROUTER'} = sha256_hex('BKPR_ROUTER'. $salt);
+    $AUTH_TOKENS{'BKPR_SYSTEM'} = sha256_hex('BKPR_SYSTEM'. $secret);
+    $AUTH_TOKENS{'BKPR_ADMIN'}  = sha256_hex('BKPR_ADMIN' . $secret);
+    $AUTH_TOKENS{'BKPR_ROUTER'} = sha256_hex('BKPR_ROUTER'. $secret);
 }
 
 sub __has_authorization_token {
@@ -235,7 +230,7 @@ sub __init_worker {
 sub on_startup {
     # Placeholder, intended to be overrided
     my $class = ref $_[0];
-    log_warn "Worker class $class doesn't define on_startup() method";
+    warn "Worker class $class doesn't define on_startup() method";
 }
 
 sub on_shutdown {
@@ -245,8 +240,8 @@ sub on_shutdown {
 sub authorize_request {
     # Placeholder, must to be overrided
     my $class = ref $_[0];
-    log_error "Worker class $class doesn't define authorize_request() method";
-    return undef; # do not authorize
+    warn "Worker class $class doesn't define authorize_request() method";
+    return undef; # do NOT authorize
 }
 
 
@@ -256,19 +251,17 @@ sub accept_notifications {
     my $worker    = $self->{_WORKER};
     my $callbacks = $worker->{callbacks};
 
-    my ($file, $line) = (caller)[1,2];
-    my $at = "at $file line $line\n";
-
     foreach my $fq_meth (keys %args) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or die "Invalid notification method '$fq_meth' $at";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid notification method $fq_meth";
 
         my ($service, $method) = ($1, $2);
 
         my $callback = $self->__get_cb_coderef($fq_meth, $args{$fq_meth});
 
-        die "Already accepting notifications '$fq_meth' $at" if exists $callbacks->{"msg.$fq_meth"};
+        #TODO: croak does not report correct caller
+        croak "Already accepting notifications $fq_meth" if exists $callbacks->{"msg.$fq_meth"};
         $callbacks->{"msg.$fq_meth"} = $callback;
 
         my $local_bus = $self->{_BUS}->{bus_role};
@@ -282,16 +275,17 @@ sub accept_notifications {
                 # ($payload_ref, $properties) = @_;
 
                 # Enqueue notification
-                push @{$worker->{task_queue_high}}, [ @_ ];
+                push @{$worker->{job_queue_high}}, [ @_ ];
 
                 unless ($worker->{queued_tasks}) {
                     $worker->{queued_tasks} = 1;
+                    $worker->{busy_since} = Time::HiRes::time;
                     AnyEvent::postpone { $self->__drain_task_queue };
                 }
             },
             on_suback => sub {
                 my ($success, $prop) = @_;
-                die "Could not subscribe to topic '$topic' $at" unless $success;
+                croak "Could not subscribe to $topic" unless $success;
             }
         );
     }
@@ -311,9 +305,7 @@ sub __get_cb_coderef {
         return \&{"${class}::${callback}"};
     }
     else {
-        my ($file, $line) = (caller(1))[1,2];
-        my $at = "at $file line $line\n";
-        die "Invalid handler '$callback' for '$method' $at";
+        croak "Invalid callback '$callback' for '$method'";
     }
 }
 
@@ -325,50 +317,49 @@ sub accept_remote_calls {
     my $callbacks = $worker->{callbacks};
     my %subscribed_to;
 
-    my ($file, $line) = (caller)[1,2];
-    my $at = "at $file line $line\n";
-
     foreach my $fq_meth (keys %args) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or die "Invalid remote call method '$fq_meth' $at";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid job method $fq_meth";
 
         my ($service, $method) = ($1, $2);
 
         my $callback = $self->__get_cb_coderef($fq_meth, $args{$fq_meth});
 
-        die "Already accepting remote calls '$fq_meth' $at" if exists $callbacks->{"req.$fq_meth"};
+        #TODO: croak does not report correct caller
+        croak "Already accepting jobs $fq_meth" if exists $callbacks->{"req.$fq_meth"};
         $callbacks->{"req.$fq_meth"} = $callback;
 
         next if $subscribed_to{$service};
         $subscribed_to{$service} = 1;
 
-        if (keys %subscribed_to == 2) {
-            log_warn "Running multiple services within a single worker hurts load balancing $at";
+        if (keys %subscribed_to > 1) {
+            carp "Running multiple services within a single worker hurts load balancing (don't do that)";
         }
 
         my $local_bus = $self->{_BUS}->{bus_role};
 
-        my $topic = "\$share/BKPR/req/$local_bus/$service";
-        $topic =~ tr|.*|/#|;
+        my $queue = "\$share/BKPR/req/$local_bus/$service";
+        $queue =~ tr|.*|/#|;
 
         $self->{_BUS}->subscribe(
-            topic       => $topic,
+            topic       => $queue,
             maximum_qos => 1,
             on_publish  => sub {
-                # ($payload_ref, $mqtt_properties) = @_;
+                # ($body_ref, $msg_headers) = @_;
 
-                # Enqueue request
-                push @{$worker->{task_queue_low}}, [ @_ ];
+                # Enqueue job
+                push @{$worker->{job_queue_low}}, [ @_ ];
 
                 unless ($worker->{queued_tasks}) {
                     $worker->{queued_tasks} = 1;
+                    $worker->{busy_since} = Time::HiRes::time;
                     AnyEvent::postpone { $self->__drain_task_queue };
                 }
             },
             on_suback => sub {
                 my ($success, $prop) = @_;
-                die "Could not subscribe to topic '$topic' $at" unless $success;
+                croak "Could not subscribe to $queue" unless $success;
             }
         );
     }
@@ -380,22 +371,14 @@ sub __drain_task_queue {
     my $self = shift;
 
     # Ensure that draining does not recurse
-    Carp::confess "Unexpected task queue processing recursion" if $_TASK_QUEUE_DEPTH;
+    die "Task queue processing is recursing" if ($_TASK_QUEUE_DEPTH);
     $_TASK_QUEUE_DEPTH++;
 
-    my $timing_tasks;
-
-    unless (defined $BUSY_SINCE) {
-        # Measure time elapsed while processing requests
-        $BUSY_SINCE = Time::HiRes::time;
-        $timing_tasks = 1; 
-    }
-
     my $worker = $self->{_WORKER};
     my $client = $self->{_CLIENT};
     my $task;
 
-    # When requests or notifications are received these are not executed immediately
+    # When jobs or notifications are received they are not executed immediately
     # because that could happen in the middle of the process of another request,
     # so these tasks get queued until the worker is ready to process the next one.
     #
@@ -406,47 +389,47 @@ sub __drain_task_queue {
 
     DRAIN: {
 
-        while ($task = shift @{$worker->{task_queue_high}}) {
+        while ($task = shift @{$worker->{job_queue_high}}) {
 
             ## Notification
 
-            my ($payload_ref, $mqtt_properties) = @$task;
+            my ($body_ref, $msg_headers) = @$task;
 
             $worker->{notif_count}++;
 
             eval {
 
-                my $request = decode_json($$payload_ref);
+                my $request = decode_json($$body_ref);
 
                 unless (ref $request eq 'HASH' && $request->{jsonrpc} eq '2.0') {
-                    log_error "Received invalid JSON-RPC 2.0 notification";
+                    log_warn "Received invalid JSON-RPC 2.0 notification";
                     return;
                 }
 
                 bless $request, 'Beekeeper::JSONRPC::Notification';
-                $request->{_mqtt_properties} = $mqtt_properties;
+                $request->{_mqtt_prop} = $msg_headers;
 
                 my $method = $request->{method};
 
                 unless (defined $method && $method =~ m/^([\.\w-]+)\.([\w-]+)$/) {
-                    log_error "Received notification with invalid method '$method'";
+                    log_warn "Received notification with invalid method $method";
                     return;
                 }
 
                 my $cb = $worker->{callbacks}->{"msg.$1.$2"} || 
                          $worker->{callbacks}->{"msg.$1.*"};
 
-                local $client->{caller_id}   = $mqtt_properties->{'clid'};
-                local $client->{caller_addr} = $mqtt_properties->{'addr'};
-                local $client->{auth_data}   = $mqtt_properties->{'auth'};
+                local $client->{caller_id}   = $msg_headers->{'clid'};
+                local $client->{caller_addr} = $msg_headers->{'addr'};
+                local $client->{auth_data}   = $msg_headers->{'auth'};
 
                 unless (($self->authorize_request($request) || "") eq BKPR_REQUEST_AUTHORIZED) {
-                    log_error "Notification '$method' was not authorized";
+                    log_warn "Notification $method was not authorized";
                     return;
                 }
 
                 unless ($cb) {
-                    log_error "No handler found for received notification '$method'";
+                    log_warn "No callback found for received notification $method";
                     return;
                 }
 
@@ -459,21 +442,21 @@ sub __drain_task_queue {
             }
         }
 
-        if ($task = shift @{$worker->{task_queue_low}}) {
+        if ($task = shift @{$worker->{job_queue_low}}) {
 
             ## RPC Call
 
-            my ($payload_ref, $mqtt_properties) = @$task;
+            my ($body_ref, $msg_headers) = @$task;
 
             $worker->{calls_count}++;
             my ($request, $request_id, $result, $response);
 
             $result = eval {
 
-                $request = decode_json($$payload_ref);
+                $request = decode_json($$body_ref);
 
                 unless (ref $request eq 'HASH' && $request->{jsonrpc} eq '2.0') {
-                    log_error "Received invalid JSON-RPC 2.0 request";
+                    log_warn "Received invalid JSON-RPC 2.0 request";
                     die Beekeeper::JSONRPC::Error->invalid_request;
                 }
 
@@ -481,36 +464,36 @@ sub __drain_task_queue {
                 my $method  = $request->{method};
 
                 bless $request, 'Beekeeper::JSONRPC::Request';
-                $request->{_mqtt_properties} = $mqtt_properties;
+                $request->{_mqtt_prop} = $msg_headers;
 
                 unless (defined $method && $method =~ m/^([\.\w-]+)\.([\w-]+)$/) {
-                    log_error "Received request with invalid method '$method'";
+                    log_warn "Received request with invalid method $method";
                     die Beekeeper::JSONRPC::Error->method_not_found;
                 }
 
                 my $cb = $worker->{callbacks}->{"req.$1.$2"} || 
                          $worker->{callbacks}->{"req.$1.*"};
 
-                local $client->{caller_id}   = $mqtt_properties->{'clid'};
-                local $client->{caller_addr} = $mqtt_properties->{'addr'};
-                local $client->{auth_data}   = $mqtt_properties->{'auth'};
+                local $client->{caller_id}   = $msg_headers->{'clid'};
+                local $client->{caller_addr} = $msg_headers->{'addr'};
+                local $client->{auth_data}   = $msg_headers->{'auth'};
 
                 unless (($self->authorize_request($request) || "") eq BKPR_REQUEST_AUTHORIZED) {
-                    log_error "Request '$method' was not authorized";
+                    log_warn "Request $method was not authorized";
                     die Beekeeper::JSONRPC::Error->request_not_authorized;
                 }
 
                 unless ($cb) {
-                    log_error "No handler found for received request '$method'";
+                    log_warn "No callback found for received request $method";
                     die Beekeeper::JSONRPC::Error->method_not_found;
                 }
 
-                # Execute method handler
+                # Execute job
                 $cb->($self, $request->{params}, $request);
             };
 
             if ($@) {
-                # Got an exception while executing method handler
+                # Got an exception while executing job
                 if (blessed($@) && $@->isa('Beekeeper::JSONRPC::Error')) {
                     # Handled exception
                     $response = $@;
@@ -527,11 +510,6 @@ sub __drain_task_queue {
                 # Explicit error response
                 $response = $result;
             }
-            elsif ($request->{_async_response}) {
-                # Response was deferred and will be sent later
-                $worker->{in_progress}++;
-                $request->{_worker} = $self;
-            }
             else {
                 # Build a success response
                 $response = {
@@ -540,9 +518,9 @@ sub __drain_task_queue {
                 };
             }
 
-            if (defined $request_id && defined $response) {
+            if ($request_id) {
 
-                # Send back response to caller
+                # Send response back to caller
 
                 $response->{id} = $request_id;
 
@@ -556,46 +534,45 @@ sub __drain_task_queue {
                     $json = $JSON->encode( $response );
                 }
 
-                # Request is acknowledged as received just after sending the response. So, if
-                # the process is abruptly interrupted here, the broker will send the request to
-                # another worker and it will be executed twice (acking the request just before
+                # Request is ack'ed as received just after sending the response. So, if the
+                # process is abruptly interrupted here, the broker will send the request to
+                # another worker and it will be executed twice! (acking the request just before 
                 # processing it may cause unprocessed requests or undelivered responses)
 
                 $self->{_BUS}->publish(
-                    topic     => $mqtt_properties->{'response_topic'},
-                    addr      => $mqtt_properties->{'addr'},
+                    topic     => $msg_headers->{'response_topic'},
+                    addr      => $msg_headers->{'addr'},
                     payload   => \$json,
                     buffer_id => 'response',
                 );
 
-                if (exists $mqtt_properties->{'packet_id'}) {
+                if (exists $msg_headers->{'packet_id'}) {
 
                     $self->{_BUS}->puback(
-                        packet_id => $mqtt_properties->{'packet_id'},
+                        packet_id => $msg_headers->{'packet_id'},
                         buffer_id => 'response',
                     );
                 }
                 else {
                     # Should not happen (clients must publish with QoS 1)
-                    log_warn "Request published with QoS 0 to topic " . $mqtt_properties->{'topic'};
+                    log_warn "Request published with QoS 0 to " . $msg_headers->{'topic'};
                 }
 
                 $self->{_BUS}->flush_buffer( buffer_id => 'response' );
             }
             else {
 
-                # Acknowledge requests that doesn't send a response right now (fire & forget calls
-                # and requests handled asynchronously), signaling the broker to send more requests
+                # Background jobs doesn't expect responses
 
                 $self->{_BUS}->puback(
-                    packet_id => $mqtt_properties->{'packet_id'},
+                    packet_id => $msg_headers->{'packet_id'},
                 );
             }
         }
 
-        redo DRAIN if (@{$worker->{task_queue_high}} || @{$worker->{task_queue_low}});
+        redo DRAIN if (@{$worker->{job_queue_high}} || @{$worker->{job_queue_low}});
 
-        # Execute tasks postponed until task queue is empty
+        # Execute tasks postponed until job queue is empty
         if (exists $worker->{postponed}) {
             $_->() foreach @{$worker->{postponed}};
             delete $worker->{postponed};
@@ -604,88 +581,30 @@ sub __drain_task_queue {
 
     $_TASK_QUEUE_DEPTH--;
 
-    if (defined $timing_tasks) {
-        $BUSY_TIME += Time::HiRes::time - $BUSY_SINCE;
-        undef $BUSY_SINCE;
-    }
+    # Measure time elapsed since request reception till 
+    $worker->{busy_time} += Time::HiRes::time - $worker->{busy_since};
+    $worker->{busy_since} = 0;
 
     $worker->{queued_tasks} = 0;
 }
 
-sub __send_response {
-    my ($self, $request, $result) = @_;
-
-    # Send back async response to caller
-
-    my ($timing_tasks, $response);
-
-    $self->{_WORKER}->{in_progress}--;
-
-    # fire & forget calls doesn't expect responses
-    return unless defined $request->{id};
-
-    unless (defined $BUSY_SINCE) {
-        $BUSY_SINCE = Time::HiRes::time;
-        $timing_tasks = 1; 
-    }
-
-    if (blessed($result) && $result->isa('Beekeeper::JSONRPC::Error')) {
-        # Explicit error response
-        $response = $result;
-    }
-    else {
-        # Build a success response
-        $response = {
-            jsonrpc => '2.0',
-            result  => $result,
-        };
-    }
-
-    $response->{id} = $request->{id};
-
-    local $@;
-    my $json = eval { $JSON->encode( $response ) };
-
-    if ($@) {
-        # Probably response contains blessed references 
-        log_error "Couldn't serialize response as JSON: $@";
-        $response = Beekeeper::JSONRPC::Error->server_error;
-        $response->{id} = $request->{id};
-        $json = $JSON->encode( $response );
-    }
-
-    $self->{_BUS}->publish(
-        topic    => $request->{_mqtt_properties}->{'response_topic'},
-        addr     => $request->{_mqtt_properties}->{'addr'},
-        payload  => \$json,
-    );
-
-    if (defined $timing_tasks) {
-        $BUSY_TIME += Time::HiRes::time - $BUSY_SINCE;
-        undef $BUSY_SINCE;
-    }
-}
-
 
 sub stop_accepting_notifications {
     my ($self, @methods) = @_;
 
-    my ($file, $line) = (caller)[1,2];
-    my $at = "at $file line $line\n";
-
-    die "No method specified $at" unless @methods;
+    croak "No method specified" unless @methods;
 
     foreach my $fq_meth (@methods) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or die "Invalid method '$fq_meth' $at";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid method $fq_meth";
 
         my ($service, $method) = ($1, $2);
 
         my $worker = $self->{_WORKER};
 
         unless (defined $worker->{callbacks}->{"msg.$fq_meth"}) {
-            log_warn "Not previously accepting notifications '$fq_meth' $at";
+            carp "Not previously accepting notifications $fq_meth";
             next;
         }
 
@@ -700,11 +619,11 @@ sub stop_accepting_notifications {
         # even then wait a bit more, as some brokers may send messages *after* unsubscription.
         my $postpone = sub {
 
-           my $unsub_tmr; $unsub_tmr = AnyEvent->timer( 
+            $worker->{_timers}->{"unsub-$topic"} = AnyEvent->timer( 
                 after => UNSUBSCRIBE_LINGER, cb => sub {
 
                     delete $worker->{callbacks}->{"msg.$fq_meth"};
-                    undef $unsub_tmr;
+                    delete $worker->{_timers}->{"unsub-$topic"};
                 }
             );
         };
@@ -714,7 +633,8 @@ sub stop_accepting_notifications {
             on_unsuback => sub {
                 my ($success, $prop) = @_;
 
-                log_error "Could not unsubscribe from topic '$topic' $at" unless $success; 
+                #TODO: Report caller of stop_accepting_notifications method
+                warn "Could not unsubscribe from $topic" unless $success; 
 
                 my $postponed = $worker->{postponed} ||= [];
                 push @$postponed, $postpone;
@@ -729,15 +649,12 @@ sub stop_accepting_notifications {
 sub stop_accepting_calls {
     my ($self, @methods) = @_;
 
-    my ($file, $line) = (caller)[1,2];
-    my $at = "at $file line $line\n";
-
-    die "No method specified $at" unless @methods;
+    croak "No method specified" unless @methods;
 
     foreach my $fq_meth (@methods) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or die "Invalid remote call method '$fq_meth' $at";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid method $fq_meth";
 
         my ($service, $method) = ($1, $2);
 
@@ -746,7 +663,7 @@ sub stop_accepting_calls {
             # through a single MQTT subscription (in order to load balance them), it is 
             # not possible to reject a single method. A workaround is to use a different
             # class for each method that need to be individually rejected.
-            die "Cannot stop accepting individual methods, only '$service.*' is allowed $at";
+            croak "Cannot cancel individual job subscription to $fq_meth";
         }
 
         my $worker    = $self->{_WORKER};
@@ -755,7 +672,8 @@ sub stop_accepting_calls {
         my @cb_keys = grep { $_ =~ m/^req.\Q$service\E\b/ } keys %$callbacks;
 
         unless (@cb_keys) {
-            log_warn "Not previously accepting remote calls '$fq_meth' $at";
+            #TODO: BUG: carp reports caller as Beekeeper/WorkerPool.pm line 440
+            carp "Not previously accepting jobs $fq_meth";
             next;
         }
 
@@ -764,42 +682,21 @@ sub stop_accepting_calls {
         my $topic = "\$share/BKPR/req/$local_bus/$service";
         $topic =~ tr|.*|/#|;
 
-        # Cannot remove callbacks right now, as new requests could be in flight or be already 
-        # queued. We must wait for unsubscription completion, and then until the task queue 
-        # is empty to ensure that all received requests were processed. And even then wait a
-        # bit more, as some brokers may send requests *after* unsubscription.
+        # Cannot remove callbacks right now, as new jobs could be in flight or be already 
+        # queued. We must wait for unsubscription completion, and then until the job queue 
+        # is empty to ensure that all received jobs were processed. And even then wait a
+        # bit more, as some brokers may send jobs *after* unsubscription.
         my $postpone = sub {
 
-            $worker->{stop_cv}->begin;
-
-            my $unsub_tmr; $unsub_tmr = AnyEvent->timer( 
+            $worker->{_timers}->{"unsub-$topic"} = AnyEvent->timer( 
                 after => UNSUBSCRIBE_LINGER, cb => sub {
 
                     delete $worker->{callbacks}->{$_} foreach @cb_keys;
                     delete $worker->{subscriptions}->{$service};
-                    undef $unsub_tmr;
+                    delete $worker->{_timers}->{"unsub-$topic"};
 
-                    return unless $worker->{shutting_down};
-
-                    if ($worker->{in_progress} > 0) {
-
-                        # The task queue is empty now, but an asynchronous method handler is
-                        # still busy processing some requests received previously. Wait for
-                        # these requests to be completed before telling _work_forever to stop
-
-                        my $wait_time = 60;
-                        $worker->{stop_cv}->begin;
-
-                        my $busy_tmr; $busy_tmr = AnyEvent->timer( after => 1, interval => 1, cb => sub {
-                            unless ($worker->{in_progress} > 0 && --$wait_time > 0) {
-                                undef $busy_tmr;
-                                $worker->{stop_cv}->end;
-                            }
-                        });
-                    }
-
-                    # Tell _work_forever to stop
-                    $worker->{stop_cv}->end;
+                    # When shutting down tell _work_forever to stop
+                    $worker->{stop_cv}->end if $worker->{shutting_down};
                 }
             );
         };
@@ -809,7 +706,8 @@ sub stop_accepting_calls {
             on_unsuback  => sub {
                 my ($success, $prop) = @_;
 
-                log_error "Could not unsubscribe from topic '$topic' $at" unless $success; 
+                #TODO: Report caller of stop_accepting_calls method
+                warn "Could not unsubscribe from $topic" unless $success; 
 
                 my $postponed = $worker->{postponed} ||= [];
                 push @$postponed, $postpone;
@@ -858,8 +756,6 @@ sub stop_working {
 
     # This is the default handler for TERM signal
 
-    $worker->{shutting_down} = 1;
-
     unless (exists $worker->{stop_cv}) {
         # Worker did not completed initialization yet
         CORE::exit(0);
@@ -871,18 +767,20 @@ sub stop_working {
         $services{$1} = 1;
     }
 
-    if (keys %services) {
-
-        # Cannot exit right now, as some requests could be in flight or already queued.
-        # So tell the broker to stop sending requests, and exit after the task queue is empty
-        foreach my $service (keys %services) {
-
-            $self->stop_accepting_calls( $service . '.*' );
-        }
-    }
-    else {
-        # Tell _work_forever to stop
+    unless (keys %services) {
         $worker->{stop_cv}->send;
+        return;
+    }
+
+    $worker->{shutting_down} = 1;
+
+    # Cannot exit right now, as some jobs could be in flight or already queued.
+    # So tell the broker to stop sending jobs, and exit after the job queue is empty
+    foreach my $service (keys %services) {
+
+        $worker->{stop_cv}->begin;
+
+        $self->stop_accepting_calls( $service . '.*' );
     }
 }
 
@@ -907,8 +805,8 @@ sub __report_status {
     $worker->{notif_count} = 0;
 
     # Average load as percentage of wall clock busy time (not cpu usage)
-    my $load = sprintf("%.2f", ($BUSY_TIME - $worker->{busy_time}) / $period * 100);
-    $worker->{busy_time} = $BUSY_TIME;
+    my $load = sprintf("%.2f", $worker->{busy_time} / $period * 100);
+    $worker->{busy_time} = 0;
 
     #ENHACEMENT: report handled and unhandled errors count
 
@@ -974,7 +872,7 @@ Beekeeper::Worker - Base class for creating services
 
 =head1 VERSION
 
-Version 0.06
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -1021,7 +919,7 @@ Base class for creating services.
 
 =head1 CONSTRUCTOR
 
-L<Beekeeper::Worker> objects are created automatically by L<Beekeeper::WorkerPool>
+C<Beekeeper::Worker> objects are created automatically by C<Beekeeper::WorkerPool>
 after spawning new processes.
 
 =head1 METHODS
@@ -1031,10 +929,10 @@ after spawning new processes.
 This method is executed on a fresh worker process immediately after it was spawned,
 after connecting to the broker and initializing the logger.
 
-The default implementation is just a placeholder, intended to be overrided in subclasses.
+It is placeholder intended to be overrided in subclasses, which in their implementation
+perform startup tasks and declare which job methods and notifications will be handled.
 
-This is the place to perform startup tasks (like creating database or cache connections)
-and declare which calls and notifications the worker will accept.
+This is the place to initialize, for example, persistent database or cache connections.
 
 After this method returns the worker will wait for incoming events to handle.
 
@@ -1046,47 +944,44 @@ It can be overrided as needed, the default implementation does nothing.
 
 =head3 authorize_request( $req )
 
-This method MUST be overrided in worker classes, as the default behavior is
+This method MUST be overrided in your worker classes, as the default behavior is
 to deny the execution of any request.
 
 When a request is received this method is called before executing the corresponding
-handler, and it must return the exported constant C<BKPR_REQUEST_AUTHORIZED> in order
-to authorize it. Returning any other value will result in the request being ignored. 
+callback, and it must return the exported constant C<BKPR_REQUEST_AUTHORIZED> in order to
+authorize it. Returning any other value will result in the request being ignored. 
 
 This is the place to handle application authentication and authorization.
 
-Parameter C<$req> is either a L<Beekeeper::JSONRPC::Notification> or a 
-L<Beekeeper::JSONRPC::Request> object.
-
 =head3 log_handler
 
-By default, all workers use a L<Beekeeper::Logger> logger which logs errors and
+By default, all workers use a C<Beekeeper::Logger> logger which logs errors and
 warnings to files and also to a topic on the message bus. The command line tool
-L<bkpr-log> allows to inspect in real time the logs from the message bus. 
+C<bkpr-log> allows to inspect in real time the logs from the message bus. 
 
-This method can be overrided in worker classes in order to replace the default log 
-mechanism for another one. To do so, the new implementation must return an object 
-implementing a C<log> method (see C<Beekeeper::Logger::log> for reference).
+To replace this default log mechanism for another one of your choice, you must 
+override the class C<log_handler> method and make that return an object implementing
+a C<log> method.
 
 For convenience you can import the ':log' symbols and expose to your class the
 functions C<log_fatal>, C<log_alert>, C<log_critical>, C<log_error>, C<log_warn>, 
 C<log_warning>, C<log_notice>, C<log_info>, C<log_debug> and C<log_trace>.
 
-These will call the underlying C<log> method of the logger class if the severity
-is equal or higher than C<$Beekeeper::Worker::LogLevel>, which is set to C<LOG_INFO>
-by default. The log level can be set to C<LOG_DEBUG> with the --debug option of L<bkpr>, 
-or setting a "debug" option to a true value in config file pool.config.json.
+These will call the underlying C<log> method of your logger class, if the severity
+is equal or higher than C<$Beekeeper::Worker::LogLevel>, which is set to allow 
+warnings by default. You can increase the log level to include debug info with 
+the --debug option of C<bkpr>, or from class config in file pool.config.json.
 
 Using these functions makes very easy to switch logging backends at a later date.
 
 All warnings and errors generated by the execution of the worker code are
-logged (unless their severity is below the current log level).
+logged, unless you specifically catch and ignore them (or raise the log level).
 
 =head3 RPC call methods
 
 In order to make RPC calls to another services, methods C<send_notification>, 
 C<call_remote>, C<call_remote_async>, C<fire_remote> and C<wait_async_calls> are 
-automatically imported from L<Beekeeper::Client>.
+automatically imported from C<Beekeeper::Client>.
 
 =head3 accept_notifications ( $method => $callback, ... )
 
@@ -1095,17 +990,17 @@ Make this worker start accepting specified notifications from message bus.
 C<$method> is a string with the format "{service_class}.{method}". A default
 or fallback handler can be specified using a wildcard as "{service_class}.*".
 
-C<$callback> is the method handler (a method name or a coderef) that will be called 
-when a notification is received. When executed, the handler will receive two parameters
-C<$params> (which contains the notification data itself) and C<$req> which is a
-L<Beekeeper::JSONRPC::Notification> object (usually redundant unless it is necessary
-to inspect the MQTT properties of the notification).
+C<$callback> is a method name or a coderef that will be called when a notification
+is received. When executed, the callback will receive two parameters C<$params> 
+(which contains the notification data itself) and C<$req> which is a
+C<Beekeeper::JSONRPC::Notification> object (usually redundant unless you need to
+inspect the request MQTT properties).
 
-Notifications are not expected to return a value. Any value returned from notification
-handlers will be ignored.
+Notifications are not expected to return a value, any value returned from its
+callback is ignored.
 
-The handler is executed within an eval block. If it dies the error will be logged
-but the worker will continue running.
+The callback is executed within an eval block, if it dies the error will be logged
+but otherwise the worker will continue running.
 
 Example:
 
@@ -1115,7 +1010,7 @@ Example:
   use base 'Beekeeper::Worker';
   
   sub on_startup {
-      my ($self) = @_;
+      my $self = shift;
       
       $self->accept_notifications(
           'foo.bar' => 'bar',       # call $self->bar       for notifications 'foo.bar'
@@ -1125,7 +1020,7 @@ Example:
   }  
   
   sub bar {
-       my ($self, $params, $req) = @_;
+       my ($self, $params, $req) = @_
        
        # $self is a MyWorker object
        # $params is a ref to the notification data
@@ -1141,17 +1036,18 @@ Make this worker start accepting specified RPC requests from message bus.
 C<$method> is a string with the format "{service_class}.{method}". A default
 or fallback handler can be specified using a wildcard as "{service_class}.*".
 
-C<$callback> is the method handler (a method name or a coderef) that will be 
-called when a request is received. When executed, the handler will receive two 
-parameters C<$params> (which contains the notification data itself) and C<$req>
-which is a L<Beekeeper::JSONRPC::Request> object.
+C<$callback> is a method name or a coderef that will be called when a request
+is received. When executed, the callback will receive two parameters C<$params> 
+(which contains the notification data itself) and C<$req> which is a
+C<Beekeeper::JSONRPC::Request> object (usually redundant unless you need to
+inspect the request MQTT properties).
 
-The value or reference returned by the handler will be sent back to the caller
-as response (unless the response is deferred with C<$req-\>async_response>).
+The value or data ref returned by the callback will be sent back to the caller
+as response.
 
-The handler is executed within an eval block. If it dies the error will be logged 
-and the caller will receive a generic error response, but the worker will continue
-running.
+The callback is executed within an eval block, if it dies the error will be 
+logged but otherwise the worker will continue running, and the caller will 
+receive a generic error response.
 
 Example:
 
@@ -1161,7 +1057,7 @@ Example:
   use base 'Beekeeper::Worker';
   
   sub on_startup {
-       my ($self) = @_;
+      my $self = shift;
       
       $self->accept_remote_calls(
           'foo.inc' => 'increment',  # call $self->increment  for requests to 'foo.inc'
@@ -1171,7 +1067,7 @@ Example:
   }
   
   sub increment {
-       my ($self, $params, $req) = @_;
+       my ($self, $params, $req) = @_
        
        # $self is a MyWorker object
        # $params is a ref to the parameters of the request
@@ -1182,32 +1078,6 @@ Example:
        return $params->{number} + 1;
   }
 
-Remote calls can be processed concurrently by means of calling C<$req-\>async_response>
-to tell Beekeeper that the response for the request will be deferred until it is
-available, freeing the worker to accept more requests. Once the response is ready, 
-it must be sent back to the caller with C<$req-\>send_response>.
-
-This handler process requests concurrently:
-
-  sub increment {
-      my ($self, $params, $req) = @_;
-  
-      my $number = $params->{number};
-  
-      $req->async_response;
-  
-      my $t; $t = AnyEvent->timer( after => 1, cb => sub {
-          undef $t;
-          $req->send_response( $number + 1 );
-      });
-  }
-
-Note that callback closures will not be executed in Beekeeper scope but in the event loop
-one, so uncatched exceptions in these closures will cause the worker to die and be respawn.
-
-Asynchronous method handlers use system resources more efficiently, but are significantly 
-harder to write and debug.
-
 =head3 stop_accepting_notifications ( $method, ... )
 
 Make this worker stop accepting specified notifications from message bus.
